__version__ = "2.0.1"

from .test_text_recognition import TestTextRecognition
from .test_speech_recognition import TestSpeechRecognition
from .test_intent_recognition import TestIntentRecognition
from .test_face_recognition import TestFaceRecognition
from .test_object_detection import TestObjectDetection
from .test_object_detection import TestObjectDetection


