__version__ = "2.0.3"

from .gpt_intent_recognition_impl import GPTIntentRecognitionImpl
from .snips_intent_recognition_impl import SNIPSIntentRecognitionImpl
from .i_intent_recognition import IIntentRecognition

